#include <iostream.h>
#include "generated/corbatest.hh"

static const char TEST_SERVER_NAME[] = "test-server-name"; 
static CORBA::Object_ptr getObjectReference(CORBA::ORB_ptr orb, const char objName[]);

using namespace org::dsrg::labs::corba::test::generated;

int main (int argc, char** argv)
{
  try 
  {
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
   
    CORBA::Object_var obj = getObjectReference(orb, TEST_SERVER_NAME);

    TextFunctions_var text_functions_ref = TextFunctions::_narrow(obj);
   
    CORBA::String_var src_str = (const char*) "simple-string"; 
    CORBA::String_var upper_str = text_functions_ref->toUpper( src_str );    
    cerr << " I sent, \"" <<  (char*)src_str << "\"." <<endl
    	 << "The TextFunctions::toUpper replied, \"" << (char*) upper_str <<"\"." << endl; 
    orb->destroy();
  }
  catch(CORBA::COMM_FAILURE& ex) 
  {
    cerr << "Caught system exception COMM_FAILURE -- unable to contact the "
         << "object." << endl;
  }
  catch(CORBA::SystemException& se) 
  {
    cerr << "Caught CORBA::SystemException." << endl;
  }
  catch(CORBA::Exception& e) 
  {
    cerr << "Caught CORBA::Exception." << endl;
  }
  catch(omniORB::fatalException& fe) 
  {
    cerr << "Caught omniORB::fatalException:" << endl;
    cerr << "  file: " << fe.file() << endl;
    cerr << "  line: " << fe.line() << endl;
    cerr << "  mesg: " << fe.errmsg() << endl;
  }
  catch(...) 
  {
    cerr << "Caught unknown exception." << endl;
  }

  return 0;

}

static CORBA::Object_ptr getObjectReference(CORBA::ORB_ptr orb, const char objName[])
{
  CosNaming::NamingContext_var rootContext;

  try 
  {
    // Obtain a reference to the root context of the Name service:
    CORBA::Object_var obj;
    obj = orb->resolve_initial_references("NameService");

    // Narrow the reference returned.
    rootContext = CosNaming::NamingContext::_narrow(obj);
    if( CORBA::is_nil(rootContext) ) 
    {
      cerr << "Failed to narrow the root naming context." << endl;
      return CORBA::Object::_nil();
    }
  }
  catch(CORBA::ORB::InvalidName& ex) 
  {
    // This should not happen!
    cerr << "Service required is invalid [does not exist]." << endl;
    return CORBA::Object::_nil();
  }

  // Create a name object, containing the name test/context:
  CosNaming::Name name;
  name.length(1);
  name[0].id   = CORBA::string_dup(objName);
  try 
  {
    // Resolve the name to an object reference.
    return rootContext->resolve(name);
  }
  catch(CosNaming::NamingContext::NotFound& ex) 
  {
    // This exception is thrown if any of the components of the
    // path [contexts or the object] aren't found:
    cerr << "Context not found." << endl;
  }
  catch(CORBA::COMM_FAILURE& ex) 
  {
    cerr << "Caught system exception COMM_FAILURE -- unable to contact the "
         << "naming service." << endl;
  }
  catch(CORBA::SystemException&) 
  {
    cerr << "Caught a CORBA::SystemException while using the naming service."
         << endl;
  }

  return CORBA::Object::_nil();
}
