package org.dsrg.labs.corba.test.client;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;

import org.dsrg.labs.corba.test.generated.TextFunctions;
import org.dsrg.labs.corba.test.generated.TextFunctionsHelper;
import org.dsrg.labs.util.Logger;
import org.dsrg.labs.corba.test.data.Const;

/**
 * @author ps
 *
 */
public class TestClient
{
	public void run( String[] argv)
	{
		try
		{
			ORB orb = ORB.init( argv, null );
			try
			{
				Logger.debug( "list_initial_services" );				
				String[] services = orb.list_initial_services();
				Logger.debug( "number of initial_services:" + services.length );
				for( int i = 0; i < services.length; i++ )
				{
					Logger.debug( "service " +  ( i + 1 ) + "/" + services.length + ": " + services[ i ] );
				}
			}
			catch(Exception e)
			{
				Logger.error( "Exception during list_initial_services: " + e );
				System.exit( 0 );
			}

      		// get the root naming context
			org.omg.CORBA.Object objRef = orb.resolve_initial_references( "NameService" );

      		// Use NamingContextExt instead of NamingContext. 
			// This is part of the Interoperable naming Service.  
			NamingContextExt ncRef = NamingContextExtHelper.narrow( objRef );

      		// Resolve the Object Reference in Naming
			Logger.debug( "Obtaining a handle to server object reference " + Const.TEST_SERVER_NAME ); 
			TextFunctions functionsImpl = TextFunctionsHelper.
						  narrow( ncRef.resolve_str( Const.TEST_SERVER_NAME ) );
			Logger.debug( "Handle is succesfuly obtained." );

			String testString = "corba-test-string"; 
			Logger.debug( "Before toUpper: " + testString );
			testString = functionsImpl.toUpper( testString );
			Logger.debug( "After toUpper: " + testString );
	   } 
	   catch (Exception e)
	   {
        Logger.error("Corba test client error:" + e.getMessage(), e) ;
 	   }
	}

  public static void main(String[] argv)
  {
		new TestClient().run( argv ); 
  }
}
