package org.dsrg.labs.corba.test.server;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import org.dsrg.labs.corba.test.generated.TextFunctions;
import org.dsrg.labs.corba.test.generated.TextFunctionsHelper;
import org.dsrg.labs.corba.test.generated.TextFunctionsOperations;
import org.dsrg.labs.corba.test.generated.TextFunctionsPOA;
import org.dsrg.labs.util.Logger;
import org.dsrg.labs.corba.test.data.Const;

/**
 * @author ps
 *
 */

public class TextFunctionsServer 
{
	public static void main(String[] args)
	{
		try
		{
			// create and initialize the ORB
			ORB orb = ORB.init( args, null );
			// get reference to rootpoa & activate the POAManager
			POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();

			// create servant and register it with the ORB
			TextFunctionsImpl functionsImpl = new TextFunctionsImpl();

			// get object reference from the servant
			org.omg.CORBA.Object ref = rootpoa.servant_to_reference(functionsImpl);
			Logger.debug( "TextFunction CORBA Reference:" + orb.object_to_string( ref ) ); 
			TextFunctions href = TextFunctionsHelper.narrow(ref);
			// get the root naming context
			org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
			// Use NamingContextExt which is part of the Interoperable
			// Naming Service (INS) specification.
			NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

			// bind the Object Reference in Naming
			NameComponent path[] = ncRef.to_name( Const.TEST_SERVER_NAME );
			ncRef.rebind(path, href);
            Logger.debug( "TextFunctions server registered with name:" + Const.TEST_SERVER_NAME );
			Logger.info("TextFunctions ready and waiting ...");
			// wait for invocations from clients
			orb.run();
		} 
		catch (Exception e) 
		{
  			Logger.error("ERROR: " + e.getMessage(), e);
		}

		Logger.debug("TextFunctions Exiting ...");
	}	
}

