package org.dsrg.labs.corba.translator.client;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CORBA.Any;
import org.omg.CORBA.Context;
import org.omg.CORBA.DynAny;
import org.omg.CORBA.DynEnum;
import org.omg.CORBA.DynStruct;
import org.omg.CORBA.NVList;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Request;
import org.omg.CORBA.StructMember;
import org.omg.CORBA.TypeCode;

import org.dsrg.labs.util.Logger;
import org.dsrg.labs.corba.translator.data.Const;
import org.dsrg.labs.corba.translator.generated.Language;
import org.dsrg.labs.corba.translator.generated.Message;
import org.dsrg.labs.corba.translator.generated.TranslatorHelper;
import org.dsrg.labs.corba.translator.generated.TranslatorOperations;

/**
 * @author bombel ps
 *
 */
public class TranslatorClient
{
	private ORB orb = null;

	public void run( String argv[])
	{
		try
		{
			if ( argv.length <= 2 )
			{
				System.out.println( "Please specify call's type. " );
				System.out.println( "static-static; Static client to Static server" );
				System.out.println( "static-dsi; DII client to static server" );				
				System.out.println( "static-2-dsi; Static client to DSI server" ); 
				System.out.println( "dii-2-dsi; DII client to DSI server" );	
				System.out.println( "dii-2-static; DII client to static server" );
				System.exit( 1 );
			}

			initORB( argv );	

			String call_type = argv[2];
			if ( call_type.equals("static-static") )
			{
				performStaticCall( argv );
			}
			else if ( call_type.equals("static-2-dsi"))
			{
				performStatic2DsiCall( argv );
			}
			else if ( call_type.equals("static-dsi"))
			{
				performStaticDsiCall( argv );
			}
			else if ( call_type.equals("dii-2-dsi"))
			{
				performDiiDsiCall( argv );
			}
			else if ( call_type.equals("dii-2-static"))
			{
				performDiiStaticCall( argv );
			}		
			else
			{
				System.out.println( "Call type is invalid!" );
				System.exit(1); 
			}	
						
	   } 
	   catch (Exception e)
	   {
	        Logger.error( "Translator client exception: " + e.getMessage(), e );
	   }
	}

	private void performStaticDsiCall( String argv[])
		throws Exception
	{
		Logger.info("Performing static call to DSI server ...... " );
		// Use NamingContextExt instead of NamingContext. This is
        // part of the Interoperable naming Service.
        NamingContextExt ncRef = getNamingContext( );

	    // resolve the Object Reference in Naming
        TranslatorOperations translator = TranslatorHelper.
              narrow( ncRef.resolve_str( Const.TRANSLATOR_DSI_OBJECT_NAME ) );

        Logger.debug( "Obtained a handle to a DSI translator object: " + translator );
		
		String translated = translator.translate( new Message( Language.polish, Language.english, "bialy" ) );	

		Logger.debug( "Bialy is translated from polish to english:" + translated  );
	}	


	private void performStatic2DsiCall( String argv[])
		throws Exception
	{
		Logger.info("Performing static call to DSI server ...... " );
		// Use NamingContextExt instead of NamingContext. This is
		// part of the Interoperable naming Service.
		NamingContextExt ncRef = getNamingContext( );

		// resolve the Object Reference in Naming
		TranslatorOperations translator = TranslatorHelper.
			  narrow( ncRef.resolve_str( Const.TRANSLATOR_DSI_OBJECT_NAME ) );

		Logger.debug( "Obtained a handle to a DSI translator object: " + translator );
		
		String translated = translator.translate2(  Const.LANG_POLISH,  Const.LANG_ENGLISH, "bialy" );	

		Logger.debug( "Bialy is translated from polish to english:" + translated  );
	}	


	private void performDiiDsiCall( String argv[] )
		throws Exception
	{
		Logger.info("Performing DII call to DSI server ...... " );
		// Use NamingContextExt instead of NamingContext. This is
        // part of the Interoperable naming Service.
        NamingContextExt ncRef = getNamingContext();

		// resolve the Object Reference in Naming
        org.omg.CORBA.Object translator = ncRef.resolve_str( Const.TRANSLATOR_DSI_OBJECT_NAME );

        // Now make a dynamic call to the doThis method.  The first step is
        // to build the argument list. In this case, there are three
        // arguments to the method, so create an NVList of length 3.  Next
        // create an Any objects to hold the value of the argument and insert
        // the desired value.  Finally, wrap the Any object with a NamedValue
        // and insert it into the NVList, specifying that it is an input
        // parameter.

        // string translate2( in long fromLang, in long toLang, in string text);

		NVList argList = orb.create_list( 3 );
        NamedValue nvArg;

        Any arg = orb.create_any();
        arg.insert_long( Const.LANG_POLISH );
        argList.add_value( "fromLang", arg, org.omg.CORBA.ARG_IN.value );

        arg = orb.create_any();
        arg.insert_long( Const.LANG_ENGLISH );
        argList.add_value( "toLang", arg, org.omg.CORBA.ARG_IN.value );

        arg = orb.create_any();
        arg.insert_string( "bialy" );
        argList.add_value( "text", arg, org.omg.CORBA.ARG_IN.value );

        // Create an Any object to hold the return value of the method and
        // wrap it in a NamedValue
        Any result = orb.create_any();
        result.insert_string( "dummy" );

        NamedValue resultVal =
        orb.create_named_value( "result", result, org.omg.CORBA.ARG_OUT.value );	
		// Get the local context from the ORB.
        // NOTE: This call does not work in Java 1.2, and returns a
        // NOT_IMPLEMENTED exception.  To make this work in Java 1.2, simply
        // remove this call to get_default_context(), and pass a null pointer
        // into the _create_request() call below.  This example should work
        // as is with any compliant Java CORBA environment, however.


        Request thisReq =
        	translator._create_request( null, "translate2", argList, resultVal );

        thisReq.invoke();

        // Get the return value from the Request object and output results.
        result = thisReq.result().value();
         
		Logger.debug( "Bialy is translated from polish to english:" + result.extract_string() );         
	}



    private void performDiiStaticCall( String argv[] )
		throws Exception
	{
		Logger.info("Performing DII call to static server ...... " );

		// Use NamingContextExt instead of NamingContext. This is
        // part of the Interoperable naming Service.
        NamingContextExt ncRef = getNamingContext(  );
		
        // resolve the Object Reference in Naming
        org.omg.CORBA.Object translator = ncRef.resolve_str( Const.TRANSLATOR_STATIC_OBJECT_NAME );

        // Now make a dynamic call to the doThis method.  The first step is
        // to build the argument list. In this case, there's three
        // arguments to the method, so create an NVList of length 3.  Next
        // create an Any objects to hold the value of the argument and insert
        // the desired value.  Finally, wrap the Any object with a NamedValue
        // and insert it into the NVList, specifying that it is an input
        // parameter.

         //string translate2( in long fromLang, in long toLang, in string text);

         NVList argList = orb.create_list( 3 );
         NamedValue nvArg;

         Any arg = orb.create_any();
         arg.insert_long( Const.LANG_POLISH );
         argList.add_value( "fromLang", arg, org.omg.CORBA.ARG_IN.value );

         arg = orb.create_any();
         arg.insert_long( Const.LANG_ENGLISH );
         argList.add_value( "toLang", arg, org.omg.CORBA.ARG_IN.value );

         arg = orb.create_any();
         arg.insert_string( "bialy" );
         argList.add_value( "text", arg, org.omg.CORBA.ARG_IN.value );

         // Create an Any object to hold the return value of the method and
         // wrap it in a NamedValue
         Any result = orb.create_any();
         result.insert_string( "dummy" );
		
		 NamedValue resultVal = orb.create_named_value( "result", result, org.omg.CORBA.ARG_OUT.value );
         // Get the local context from the ORB.
         // NOTE: This call does not work in Java 1.2, and returns a
         // NOT_IMPLEMENTED exception.  To make this work in Java 1.2, simply
         // remove this call to get_default_context(), and pass a null pointer
         // into the _create_request() call below.  This example should work
         // as is with any compliant Java CORBA environment, however.

         Request thisReq = translator._create_request( null, "translate2", argList, resultVal );

         thisReq.invoke();

         // Get the return value from the Request object and output results.
         result = thisReq.result().value();
         Logger.debug("translate2 returned: " + result.extract_string() );
	} 

	private void performStaticCall( String argv[] )
		throws Exception
	{
		  Logger.info("Performing static call to static server ...... " );
 
	      // Use NamingContextExt instead of NamingContext. This is
          // part of the Interoperable naming Service.
          NamingContextExt ncRef = getNamingContext(  );

            // resolve the Object Reference in Naming
          TranslatorOperations translator = TranslatorHelper.
              narrow( ncRef.resolve_str( Const.TRANSLATOR_STATIC_OBJECT_NAME ) );

          Logger.debug( "Obtained a handle  to a static translator object: " + translator );

          Message[] messages = new Message[]
          {
              new Message( Language.polish, Language.english, "zielony" ),
              new Message( Language.polish, Language.english, "bialy" ),
              new Message( Language.polish, Language.english, "czarny" ),
              new Message( Language.polish, Language.english, "niebieski" ),
              new Message( Language.english, Language.polish, "yellow" )
          };
          for( int i = 0; i < messages.length; i++ )
          {
                Logger.debug( messages[i].text + " -> " + translator.translate( messages[ i ] ) );
          }

	}

	private NamingContextExt getNamingContext( )
		throws Exception
	{
		// get the root naming context
        org.omg.CORBA.Object objRef = orb.resolve_initial_references( "NameService" );

        // Use NamingContextExt instead of NamingContext. 
		// This is part of the Interoperable naming Service.
        return NamingContextExtHelper.narrow( objRef );
	}

	private void initORB( String argv[] )
		throws Exception
	{
		orb = ORB.init( argv, null );
	}

    public static void main(String argv[] )
	{
		new TranslatorClient().run(argv);
	}
}
