package org.dsrg.labs.corba.translator.server;

import java.util.Hashtable;

import org.omg.CORBA.ARG_IN;
import org.omg.CORBA.Any;
import org.omg.CORBA.BAD_OPERATION;
import org.omg.CORBA.Context;
import org.omg.CORBA.ContextList;
import org.omg.CORBA.DomainManager;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.NO_IMPLEMENT;
import org.omg.CORBA.NVList;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.Policy;
import org.omg.CORBA.Request;
import org.omg.CORBA.ServerRequest;
import org.omg.CORBA.SetOverrideType;
import org.omg.CORBA.TCKind;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.DynamicImplementation;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import org.dsrg.labs.util.Logger;
import org.dsrg.labs.corba.translator.data.Const;
import org.dsrg.labs.corba.translator.generated.Language;
import org.dsrg.labs.corba.translator.generated.Message;
import org.dsrg.labs.corba.translator.generated.Translator;
import org.dsrg.labs.corba.translator.generated.TranslatorHelper;
import org.dsrg.labs.corba.translator.generated.TranslatorPOA;

/**
 * @author ps
 *
 */
public class TranslatorDSIImpl extends DynamicImplementation
{
	private String[] _ids =	{
		"IDL:org/dsrg/labs/corba/translator/generated/Translator:1.0",
		"IDL:omg.org/CORBA/Object:1.0"	};
	
	private static ORB orb; 
	
	public static Hashtable _polishToEnglish = new Hashtable(); 
	public static Language[] _languages = new Language[]
	{
		Language.polish,
		Language.english,
		Language.suahili
	};  
	
	public TranslatorDSIImpl()
	{
		_polishToEnglish.put( "bialy", "white" );
		_polishToEnglish.put( "czarny", "black" );
		_polishToEnglish.put( "zielony", "green" );
	}

	public String[] _all_interfaces( POA poa, byte[] objectID ) 
	{
	  return _ids;
	} 
	
	public void translate2( ServerRequest request )
	{
		// Create a new Name/Value list to be used by this request
		NVList argList = orb.create_list( 3 );
		
		// Set up the list with the expected argument details
 
 		// fromLang 
		Any arg1 = orb.create_any();
		arg1.type( orb.get_primitive_tc( TCKind.tk_long ) );
		argList.add_value( null, arg1, ARG_IN.value ); 		
		
		// toLang
		Any arg2 = orb.create_any();
		arg2.type( orb.get_primitive_tc( TCKind.tk_long ) );
		argList.add_value( null, arg2, ARG_IN.value );		
		
		// text
		Any arg3 = orb.create_any();
		arg3.type( orb.get_primitive_tc( TCKind.tk_string ) );
		argList.add_value( null, arg3, ARG_IN.value );		
		
		// Now bind all the data held in the ServerRequest to the
		// arguments in the nv list.
		// After this call the argList will contain all the values
		// sent by the client's request 		
		request.arguments( argList );

		// Extract values			
		int fromLang = arg1.extract_long();			
		int toLang = arg2.extract_long();
		String text = arg3.extract_string();
		
		// Translate
		String tranlatedText = translate2( fromLang, toLang, text );

		// Set and return result				
		Any result = orb.create_any();
		result.insert_string( tranlatedText ); 
		request.set_result( result ); 		
	}	
	
	public String translate2( int fromLang, int toLang, String text )
	{
		try
		{
			return translate( new Message( _languages[ fromLang ], _languages[ toLang ], text ) );
		}
		catch( Exception e )
		{
			Logger.error( "Server error"  + e.getMessage() , e);;
		}		
		return null;
	}
	
	public String translate(Message text)
	{
		if ( text.fromLang != Language.polish || text.toLang != Language.english )
		{
			return "Error: only polish to english translation supported";
		}
		
		try
		{
			String result = (String)_polishToEnglish.get( text.text ); 
			if ( result != null )
			{
				return result; 				
			}
		}
		catch( NullPointerException ignore )
		{
		}
		return "Error: word not found";					
	}

	public void invoke( ServerRequest request ) 
	{
		String op = request.operation();
		if ( op.equals( "translate2" ) ) 
		{
		  	translate2( request );
		} 
		else 
		{
		  	throw new BAD_OPERATION( op + " not implemented" );
		}
	}
   

	public static void main(String[] args)
	{
		try
		{
			// create and initialize the ORB
			orb = ORB.init( args, null );
			// get reference to rootpoa & activate the POAManager
			POA rootpoa = POAHelper.narrow( orb.resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();
	
			// create servant and register it with the ORB
			TranslatorDSIImpl translatorImpl = new TranslatorDSIImpl();
	
			// get object reference from the servant
			org.omg.CORBA.Object ref = rootpoa.servant_to_reference(translatorImpl);
				
			Logger.debug( "TranslatorDSI ref:" + orb.object_to_string( ref ) );
				
			Translator href = TranslatorHelper.narrow(ref);
			// get the root naming context
			org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
			// Use NamingContextExt which is part of the Interoperable
			// Naming Service (INS) specification.
			NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);
	
			// bind the Object Reference in Naming
			NameComponent path[] = ncRef.to_name( Const.TRANSLATOR_DSI_OBJECT_NAME );
			ncRef.rebind( path, href );
	
			Logger.info("Translator DSI ready and waiting ...");
			// wait for invocations from clients
			orb.run();
		} 
		catch( Exception e ) 
		{
			Logger.error( "ERROR: " + e, e );
		}

		Logger.debug( "Translator Exiting ..." );
	}
}
