package org.dsrg.labs.corba.translator.server;

import java.util.Hashtable;

import org.omg.CORBA.Context;
import org.omg.CORBA.ContextList;
import org.omg.CORBA.DomainManager;
import org.omg.CORBA.ExceptionList;
import org.omg.CORBA.NVList;
import org.omg.CORBA.NamedValue;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.Policy;
import org.omg.CORBA.Request;
import org.omg.CORBA.SetOverrideType;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import org.dsrg.labs.util.Logger;
import org.dsrg.labs.corba.translator.data.Const;
import org.dsrg.labs.corba.translator.generated.Language;
import org.dsrg.labs.corba.translator.generated.Message;
import org.dsrg.labs.corba.translator.generated.Translator;
import org.dsrg.labs.corba.translator.generated.TranslatorHelper;
import org.dsrg.labs.corba.translator.generated.TranslatorPOA;

/**
 * @author ps
 *
 */
public class TranslatorImpl extends TranslatorPOA
{
	public static Hashtable _polishToEnglish = new Hashtable(); 
	public static Language[] _languages = new Language[]
	{
		Language.polish,
		Language.english,
		Language.suahili
	};  
	
	public TranslatorImpl()
	{
		_polishToEnglish.put( "bialy", "white" );
		_polishToEnglish.put( "czarny", "black" );
		_polishToEnglish.put( "zielony", "green" );
	}
	
	public String translate2( int fromLang, int toLang, String text )
	{
		try
		{
			return translate( new Message( _languages[ fromLang ], _languages[ toLang ], text ) );
		}
		catch( Exception e )
		{
			Logger.error("Skeleton error " + e.getMessage() ,e );
		}
		return null;		
	}
	
	public String translate(Message text)
	{
		if ( text.fromLang != Language.polish || text.toLang != Language.english )
		{
			return "Error: only polish to english translation supported";
		}
		
		try
		{
			String result = (String)_polishToEnglish.get( text.text ); 
			if ( result != null )
			{
				return result; 				
			}
		}
		catch( NullPointerException ignore )
		{
		}
		return "Error: word not found";					
	}

	public static void main(String[] args)
	{
		try
		{
			// create and initialize the ORB
			ORB orb = ORB.init( args, null );
			// get reference to rootpoa & activate the POAManager
			POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();
	
			// create servant and register it with the ORB
			TranslatorImpl translatorImpl = new TranslatorImpl();
	
			// get object reference from the servant
			org.omg.CORBA.Object ref = rootpoa.servant_to_reference(translatorImpl);
				
			Logger.debug( "Translator STATIC object  ref:" + orb.object_to_string( ref ) );
				
			Translator href = TranslatorHelper.narrow(ref);
			// get the root naming context
			org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
			// Use NamingContextExt which is part of the Interoperable
			// Naming Service (INS) specification.
			NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);
	
			// bind the Object Reference in Naming
			NameComponent path[] = ncRef.to_name( Const.TRANSLATOR_STATIC_OBJECT_NAME );
			ncRef.rebind(path, href);
	
			Logger.info("Translator ready and waiting ...");
			// wait for invocations from clients
			orb.run();
		} 
		catch (Exception e) 
		{
             Logger.error(" TranslatorImpl error " + e.getMessage(), e);	
    	}

	}
}
